@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = "5.5.post555"
+__version__ = "6.5.10"
 
 from .api.opencti_api_client import OpenCTIApiClient
 from .api.opencti_api_connector import OpenCTIApiConnector
@@ -9,13 +9,18 @@ from .connector.opencti_connector_helper import (
     OpenCTIConnectorHelper,
     get_config_variable,
 )
+from .connector.opencti_metric_handler import OpenCTIMetricHandler
 from .entities.opencti_attack_pattern import AttackPattern
 from .entities.opencti_campaign import Campaign
-from .entities.opencti_case import Case
+from .entities.opencti_case_incident import CaseIncident
+from .entities.opencti_case_rfi import CaseRfi
+from .entities.opencti_case_rft import CaseRft
+from .entities.opencti_channel import Channel
 from .entities.opencti_course_of_action import CourseOfAction
 from .entities.opencti_data_component import DataComponent
 from .entities.opencti_data_source import DataSource
 from .entities.opencti_external_reference import ExternalReference
+from .entities.opencti_feedback import Feedback
 from .entities.opencti_grouping import Grouping
 from .entities.opencti_identity import Identity
 from .entities.opencti_incident import Incident
@@ -26,6 +31,7 @@ from .entities.opencti_kill_chain_phase import KillChainPhase
 from .entities.opencti_label import Label
 from .entities.opencti_location import Location
 from .entities.opencti_malware import Malware
+from .entities.opencti_malware_analysis import MalwareAnalysis
 from .entities.opencti_marking_definition import MarkingDefinition
 from .entities.opencti_note import Note
 from .entities.opencti_observed_data import ObservedData
@@ -33,23 +39,42 @@ from .entities.opencti_opinion import Opinion
 from .entities.opencti_report import Report
 from .entities.opencti_stix_core_relationship import StixCoreRelationship
 from .entities.opencti_stix_cyber_observable import StixCyberObservable
-from .entities.opencti_stix_cyber_observable_relationship import (
-    StixCyberObservableRelationship,
-)
 from .entities.opencti_stix_domain_object import StixDomainObject
+from .entities.opencti_stix_nested_ref_relationship import StixNestedRefRelationship
 from .entities.opencti_stix_object_or_stix_relationship import (
     StixObjectOrStixRelationship,
 )
 from .entities.opencti_stix_sighting_relationship import StixSightingRelationship
+from .entities.opencti_task import Task
 from .entities.opencti_threat_actor import ThreatActor
+from .entities.opencti_threat_actor_group import ThreatActorGroup
+from .entities.opencti_threat_actor_individual import ThreatActorIndividual
 from .entities.opencti_tool import Tool
 from .entities.opencti_vulnerability import Vulnerability
 from .utils.constants import (
-    MultipleStixCyberObservableRelationship,
+    CustomObjectCaseIncident,
+    CustomObjectChannel,
+    CustomObjectTask,
+    CustomObservableBankAccount,
+    CustomObservableCredential,
+    CustomObservableCryptocurrencyWallet,
+    CustomObservableHostname,
+    CustomObservableMediaContent,
+    CustomObservablePaymentCard,
+    CustomObservablePhoneNumber,
+    CustomObservableText,
+    CustomObservableTrackingNumber,
+    CustomObservableUserAgent,
+    MultipleRefRelationship,
     StixCyberObservableTypes,
     StixMetaTypes,
 )
-from .utils.opencti_stix2 import OpenCTIStix2
+from .utils.opencti_stix2 import (
+    STIX_EXT_MITRE,
+    STIX_EXT_OCTI,
+    STIX_EXT_OCTI_SCO,
+    OpenCTIStix2,
+)
 from .utils.opencti_stix2_splitter import OpenCTIStix2Splitter
 from .utils.opencti_stix2_update import OpenCTIStix2Update
 from .utils.opencti_stix2_utils import OpenCTIStix2Utils
@@ -57,12 +82,17 @@ from .utils.opencti_stix2_utils import OpenCTIStix2Utils
 __all__ = [
     "AttackPattern",
     "Campaign",
-    "Case",
+    "CaseIncident",
+    "CaseRfi",
+    "CaseRft",
+    "Channel",
+    "Task",
     "ConnectorType",
     "CourseOfAction",
     "DataComponent",
     "DataSource",
     "ExternalReference",
+    "Feedback",
     "Grouping",
     "Identity",
     "Incident",
@@ -73,6 +103,7 @@ __all__ = [
     "Label",
     "Location",
     "Malware",
+    "MalwareAnalysis",
     "MarkingDefinition",
     "Note",
     "ObservedData",
@@ -81,6 +112,7 @@ __all__ = [
     "OpenCTIApiWork",
     "OpenCTIConnector",
     "OpenCTIConnectorHelper",
+    "OpenCTIMetricHandler",
     "OpenCTIStix2",
     "OpenCTIStix2Splitter",
     "OpenCTIStix2Update",
@@ -89,15 +121,34 @@ __all__ = [
     "Report",
     "StixCoreRelationship",
     "StixCyberObservable",
-    "StixCyberObservableRelationship",
+    "StixNestedRefRelationship",
     "StixCyberObservableTypes",
     "StixDomainObject",
     "StixMetaTypes",
-    "MultipleStixCyberObservableRelationship",
+    "MultipleRefRelationship",
     "StixObjectOrStixRelationship",
     "StixSightingRelationship",
     "ThreatActor",
+    "ThreatActorGroup",
+    "ThreatActorIndividual",
     "Tool",
     "Vulnerability",
     "get_config_variable",
+    "CustomObjectCaseIncident",
+    "CustomObjectTask",
+    "CustomObjectChannel",
+    "StixCyberObservableTypes",
+    "CustomObservableCredential",
+    "CustomObservableHostname",
+    "CustomObservableUserAgent",
+    "CustomObservableBankAccount",
+    "CustomObservableCryptocurrencyWallet",
+    "CustomObservablePaymentCard",
+    "CustomObservablePhoneNumber",
+    "CustomObservableTrackingNumber",
+    "CustomObservableText",
+    "CustomObservableMediaContent",
+    "STIX_EXT_MITRE",
+    "STIX_EXT_OCTI_SCO",
+    "STIX_EXT_OCTI",
 ]
