@@ -3,7 +3,7 @@ import base64
 import datetime
 import io
 import json
-from typing import Union
+from typing import Dict, Tuple, Union
 
 import magic
 import requests
@@ -83,13 +83,13 @@ class OpenCTIApiClient:
     :param log_level: log level for the client
     :type log_level: str, optional
     :param ssl_verify: Requiring the requests to verify the TLS certificate at the server.
-    :type ssl_verify: bool, optional
+    :type ssl_verify: bool, str, optional
     :param proxies:
     :type proxies: dict, optional, The proxy configuration, would have `http` and `https` attributes. Defaults to {}
         ```
         proxies: {
-            "http: "http://my_proxy:8080"
-            "https: "http://my_proxy:8080"
+            "http": "http://my_proxy:8080"
+            "https": "http://my_proxy:8080"
         }
         ```
     :param json_logging: format the logs as json if set to True
@@ -102,14 +102,14 @@ class OpenCTIApiClient:
 
     def __init__(
         self,
-        url,
-        token,
+        url: str,
+        token: str,
         log_level="info",
-        ssl_verify=False,
-        proxies=None,
+        ssl_verify: Union[bool, str] = False,
+        proxies: Union[Dict[str, str], None] = None,
         json_logging=False,
         bundle_send_to_queue=True,
-        cert=None,
+        cert: Union[str, Tuple[str, str], None] = None,
         auth=None,
         perform_health_check=True,
     ):
@@ -213,6 +213,9 @@ class OpenCTIApiClient:
     def set_event_id(self, event_id):
         self.request_headers["opencti-event-id"] = event_id
 
+    def set_draft_id(self, draft_id):
+        self.request_headers["opencti-draft-id"] = draft_id
+
     def set_synchronized_upsert_header(self, synchronized):
         self.request_headers["synchronized-upsert"] = (
             "true" if synchronized is True else "false"
@@ -221,8 +224,11 @@ class OpenCTIApiClient:
     def set_previous_standard_header(self, previous_standard):
         self.request_headers["previous-standard"] = previous_standard
 
-    def get_request_headers(self):
-        return self.request_headers
+    def get_request_headers(self, hide_token=True):
+        request_headers_copy = self.request_headers.copy()
+        if hide_token and "Authorization" in request_headers_copy:
+            request_headers_copy["Authorization"] = "*****"
+        return request_headers_copy
 
     def set_retry_number(self, retry_number):
         self.request_headers["opencti-retry-number"] = (
@@ -274,7 +280,7 @@ class OpenCTIApiClient:
                 if is_multiple_files:
                     # [(var_name + "." + i)] if is_multiple_files else
                     for _ in file_var_item["file"]:
-                        file_vars[str(map_index)] = [(var_name + "." + str(map_index))]
+                        file_vars[str(map_index)] = [var_name + "." + str(map_index)]
                         map_index += 1
                 else:
                     file_vars[str(map_index)] = [var_name]
@@ -330,6 +336,7 @@ class OpenCTIApiClient:
                 verify=self.ssl_verify,
                 cert=self.cert,
                 proxies=self.proxies,
+                timeout=300,
             )
         # If no
         else:
@@ -340,6 +347,7 @@ class OpenCTIApiClient:
                 verify=self.ssl_verify,
                 cert=self.cert,
                 proxies=self.proxies,
+                timeout=300,
             )
         # Build response
         if r.status_code == 200:
@@ -383,6 +391,7 @@ class OpenCTIApiClient:
             verify=self.ssl_verify,
             cert=self.cert,
             proxies=self.proxies,
+            timeout=300,
         )
         if binary:
             if serialize:
@@ -402,7 +411,7 @@ class OpenCTIApiClient:
             self.app_logger.info("Health check (platform version)...")
             test = self.query(
                 """
-                  query {
+                  query healthCheck {
                     about {
                       version
                     }
@@ -663,6 +672,34 @@ class OpenCTIApiClient:
             self.app_logger.error("[upload] Missing parameter: file_name")
             return None
 
+    def create_draft(self, **kwargs):
+        """create a draft in OpenCTI API
+        :param `**kwargs`: arguments for file name creating draft (required: `draft_name`)
+        :return: returns the query response for the draft creation
+        :rtype: id
+        """
+
+        draft_name = kwargs.get("draft_name", None)
+        entity_id = kwargs.get("entity_id", None)
+
+        if draft_name is not None:
+            self.app_logger.info("Creating a draft.")
+            query = """
+                    mutation draftWorkspaceAdd($input: DraftWorkspaceAddInput!) {
+                        draftWorkspaceAdd(input: $input) {
+                            id
+                        }
+                    }
+                 """
+            queryResult = self.query(
+                query,
+                {"input": {"name": draft_name, "entity_id": entity_id}},
+            )
+            return queryResult["data"]["draftWorkspaceAdd"]["id"]
+        else:
+            self.app_logger.error("[create_draft] Missing parameter: draft_name")
+            return None
+
     def upload_pending_file(self, **kwargs):
         """upload a file to OpenCTI API
 
@@ -675,12 +712,13 @@ class OpenCTIApiClient:
         data = kwargs.get("data", None)
         mime_type = kwargs.get("mime_type", "text/plain")
         entity_id = kwargs.get("entity_id", None)
+        file_markings = kwargs.get("file_markings", [])
 
         if file_name is not None:
             self.app_logger.info("Uploading a file.")
             query = """
-                    mutation UploadPending($file: Upload!, $entityId: String) {
-                        uploadPending(file: $file, entityId: $entityId) {
+                    mutation UploadPending($file: Upload!, $entityId: String, $file_markings: [String!]) {
+                        uploadPending(file: $file, entityId: $entityId, file_markings: $file_markings) {
                             id
                             name
                         }
@@ -694,12 +732,47 @@ class OpenCTIApiClient:
                     mime_type = magic.from_file(file_name, mime=True)
             return self.query(
                 query,
-                {"file": (File(file_name, data, mime_type)), "entityId": entity_id},
+                {
+                    "file": (File(file_name, data, mime_type)),
+                    "entityId": entity_id,
+                    "file_markings": file_markings,
+                },
             )
         else:
             self.app_logger.error("[upload] Missing parameter: file_name")
             return None
 
+    def send_bundle_to_api(self, **kwargs):
+        """Push a bundle to a queue through OpenCTI API
+
+        :param `**kwargs`: arguments for bundle push (required: `connectorId` and `bundle`)
+        :return: returns the query response for the bundle push
+        :rtype: dict
+        """
+
+        connector_id = kwargs.get("connector_id", None)
+        work_id = kwargs.get("work_id", None)
+        bundle = kwargs.get("bundle", None)
+
+        if connector_id is not None and bundle is not None:
+            self.app_logger.info(
+                "Pushing a bundle to queue through API", {connector_id}
+            )
+            mutation = """
+                    mutation StixBundlePush($connectorId: String!, $bundle: String!, $work_id: String) {
+                        stixBundlePush(connectorId: $connectorId, bundle: $bundle, work_id: $work_id)
+                    }
+                 """
+            return self.query(
+                mutation,
+                {"connectorId": connector_id, "bundle": bundle, "work_id": work_id},
+            )
+        else:
+            self.app_logger.error(
+                "[bundle push] Missing parameter: connector_id or bundle"
+            )
+            return None
+
     def get_stix_content(self, id):
         """get the STIX content of any entity
 
