@@ -1,12 +1,11 @@
 # coding: utf-8
 
+import datetime
 import json
 import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class Grouping:
     def __init__(self, opencti):
@@ -35,13 +34,9 @@ class Grouping:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -53,29 +48,26 @@ class Grouping:
                     x_opencti_lastname
                 }
             }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -90,19 +82,6 @@ class Grouping:
                         external_id
                         created
                         modified
-                        importFiles {
-                            edges {
-                                node {
-                                    id
-                                    name
-                                    size
-                                    metaData {
-                                        mimetype
-                                        version
-                                    }
-                                }
-                            }
-                        }
                     }
                 }
             }
@@ -114,7 +93,7 @@ class Grouping:
             description
             context
             x_opencti_aliases
-            objects {
+            objects(all: true) {
                 edges {
                     node {
                         ... on BasicObject {
@@ -190,6 +169,201 @@ class Grouping:
                         ... on Incident {
                             name
                         }
+                        ... on StixCyberObservable {
+                            observable_value
+                        }                        
+                        ... on StixCoreRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                            relationship_type
+                        }
+                       ... on StixSightingRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }
+                    }
+                }
+            }
+        """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    created
+                    modified
+                    objectLabel {
+                        id
+                        value
+                        color
+                    }
+                }
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
+                edges {
+                    node {
+                        id
+                        standard_id
+                        entity_type
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
+                        created
+                        modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            revoked
+            confidence
+            created
+            modified
+            name
+            description
+            context
+            x_opencti_aliases
+            objects(all: true) {
+                edges {
+                    node {
+                        ... on BasicObject {
+                            id
+                            entity_type
+                            parent_types
+                        }
+                        ... on BasicRelationship {
+                            id
+                            entity_type
+                            parent_types
+                        }
+                        ... on StixObject {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }
+                        ... on AttackPattern {
+                            name
+                        }
+                        ... on Campaign {
+                            name
+                        }
+                        ... on CourseOfAction {
+                            name
+                        }
+                        ... on Individual {
+                            name
+                        }
+                        ... on Organization {
+                            name
+                        }
+                        ... on Sector {
+                            name
+                        }
+                        ... on System {
+                            name
+                        }
+                        ... on Indicator {
+                            name
+                        }
+                        ... on Infrastructure {
+                            name
+                        }
+                        ... on IntrusionSet {
+                            name
+                        }
+                        ... on Position {
+                            name
+                        }
+                        ... on City {
+                            name
+                        }
+                        ... on Country {
+                            name
+                        }
+                        ... on Region {
+                            name
+                        }
+                        ... on Malware {
+                            name
+                        }
+                        ... on ThreatActor {
+                            name
+                        }
+                        ... on Tool {
+                            name
+                        }
+                        ... on Vulnerability {
+                            name
+                        }
+                        ... on Incident {
+                            name
+                        }
+                        ... on StixCyberObservable {
+                            observable_value
+                        }                        
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -222,14 +396,23 @@ class Grouping:
         """
 
     @staticmethod
-    def generate_id(name, context):
+    def generate_id(name, context, created=None):
         name = name.lower().strip()
         context = context.lower().strip()
-        data = {"name": name, "context": context}
+        if isinstance(created, datetime.datetime):
+            created = created.isoformat()
+        if created is None:
+            data = {"name": name, "context": context}
+        else:
+            data = {"name": name, "context": context, "created": created}
         data = canonicalize(data, utf8=False)
         id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
         return "grouping--" + id
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return Grouping.generate_id(data["name"], data["context"], data["created"])
+
     """
         List Grouping objects
 
@@ -250,18 +433,23 @@ class Grouping:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 100
+        with_files = kwargs.get("withFiles", False)
 
-        LOGGER.info("Listing Groupings with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Groupings with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-            query Groupings($filters: [GroupingsFiltering!], $search: String, $first: Int, $after: ID, $orderBy: GroupingsOrdering, $orderMode: OrderingMode) {
+            query Groupings($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: GroupingsOrdering, $orderMode: OrderingMode) {
                 groupings(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -293,7 +481,7 @@ class Grouping:
             final_data = final_data + data
             while result["data"]["groupings"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["groupings"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Groupings after " + after)
+                self.opencti.app_logger.info("Listing Groupings", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -325,8 +513,9 @@ class Grouping:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            LOGGER.info("Reading Grouping {%s}.", id)
+            self.opencti.app_logger.info("Reading Grouping", {"id": id})
             query = (
                 """
                 query Grouping($id: String!) {
@@ -335,7 +524,7 @@ class Grouping:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -370,10 +559,14 @@ class Grouping:
             object_result = self.read(id=stix_id, customAttributes=custom_attributes)
         if object_result is None and name is not None and context is not None:
             object_result = self.read(
-                filters=[
-                    {"key": "name", "values": [name]},
-                    {"key": "context", "values": [context]},
-                ],
+                filters={
+                    "mode": "and",
+                    "filters": [
+                        {"key": "name", "values": [name]},
+                        {"key": "context", "values": [context]},
+                    ],
+                    "filterGroups": [],
+                },
                 customAttributes=custom_attributes,
             )
         return object_result
@@ -392,9 +585,9 @@ class Grouping:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Checking StixObjectOrStixRelationship {%s} in Grouping {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Checking StixObjectOrStixRelationship in Grouping",
+                {"id": stix_object_or_stix_relationship_id, "grouping": id},
             )
             query = """
                 query GroupingContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -410,7 +603,7 @@ class Grouping:
             )
             return result["data"]["groupingContainsStixObjectOrStixRelationship"]
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_grouping] Missing parameters: id or stixObjectOrStixRelationshipId"
             )
 
@@ -435,14 +628,15 @@ class Grouping:
         modified = kwargs.get("modified", None)
         name = kwargs.get("name", None)
         context = kwargs.get("context", None)
-        description = kwargs.get("description", "")
+        description = kwargs.get("description", None)
         x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
-        if name is not None and description is not None and context is not None:
-            LOGGER.info("Creating Grouping {%s}.", name)
+        if name is not None and context is not None:
+            self.opencti.app_logger.info("Creating Grouping", {"name": name})
             query = """
                 mutation GroupingAdd($input: GroupingAddInput!) {
                     groupingAdd(input: $input) {
@@ -474,13 +668,14 @@ class Grouping:
                         "description": description,
                         "x_opencti_aliases": x_opencti_aliases,
                         "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "x_opencti_workflow_id": x_opencti_workflow_id,
                         "update": update,
                     }
                 },
             )
             return self.opencti.process_multiple_fields(result["data"]["groupingAdd"])
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_grouping] Missing parameters: name and description and context"
             )
 
@@ -498,12 +693,12 @@ class Grouping:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Adding StixObjectOrStixRelationship {%s} to Grouping {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Adding StixObjectOrStixRelationship to Grouping",
+                {"id": stix_object_or_stix_relationship_id, "grouping": id},
             )
             query = """
-               mutation GroupingEditRelationAdd($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation GroupingEditRelationAdd($id: ID!, $input: StixRefRelationshipAddInput!) {
                    groupingRelationAdd(id: $id, input: $input) {
                         id
                    }
@@ -521,7 +716,7 @@ class Grouping:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_grouping] Missing parameters: id and stixObjectOrStixRelationshipId",
             )
             return False
@@ -540,9 +735,9 @@ class Grouping:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Removing StixObjectOrStixRelationship {%s} to {%s}",
-                *(stix_object_or_stix_relationship_id, Grouping),
+            self.opencti.app_logger.info(
+                "Removing StixObjectOrStixRelationship to Grouping",
+                {"id": stix_object_or_stix_relationship_id, "grouping": id},
             )
             query = """
                mutation GroupingEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -561,7 +756,7 @@ class Grouping:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_grouping] Missing parameters: id and stixObjectOrStixRelationshipId",
             )
             return False
@@ -580,55 +775,74 @@ class Grouping:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_aliases" not in stix_object:
-                stix_object[
-                    "x_opencti_aliases"
-                ] = self.opencti.get_attribute_in_extension("aliases", stix_object)
+                stix_object["x_opencti_aliases"] = (
+                    self.opencti.get_attribute_in_extension("aliases", stix_object)
+                )
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                 )
 
             return self.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
                 objects=extras["object_ids"] if "object_ids" in extras else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=stix_object["name"],
                 context=stix_object["context"],
-                description=self.opencti.stix2.convert_markdown(
-                    stix_object["description"]
-                )
-                if "description" in stix_object
-                else "",
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
-                objectOrganization=stix_object["granted_refs"]
-                if "granted_refs" in stix_object
-                else None,
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_object["description"])
+                    if "description" in stix_object
+                    else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
                 x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_grouping] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_grouping] Missing parameters: stixObject"
+            )
