@@ -1,6 +1,9 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
 
 from pycti.utils.constants import IdentityTypes
 
@@ -23,26 +26,23 @@ class Identity:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
                     contact_information
                     x_opencti_aliases
+                    x_opencti_reliability
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
-                    }                    
+                        id
+                        value
+                        color
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
-                    x_opencti_reliability
                 }
                 ... on Individual {
                     x_opencti_firstname
@@ -50,28 +50,25 @@ class Identity:
                 }
             }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -97,6 +94,7 @@ class Identity:
             name
             description
             x_opencti_aliases
+            x_opencti_reliability
             contact_information
             ... on Individual {
                 x_opencti_firstname
@@ -104,9 +102,138 @@ class Identity:
             }
             ... on Organization {
                 x_opencti_organization_type
-                x_opencti_reliability
             }
         """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    x_opencti_reliability
+                    created
+                    modified
+                    objectLabel {
+                        id
+                        value
+                        color
+                    }
+                }
+                ... on Organization {
+                    x_opencti_organization_type
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
+                edges {
+                    node {
+                        id
+                        standard_id
+                        entity_type
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
+                        created
+                        modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            revoked
+            confidence
+            created
+            modified
+            identity_class
+            name
+            description
+            x_opencti_aliases
+            x_opencti_reliability
+            contact_information
+            ... on Individual {
+                x_opencti_firstname
+                x_opencti_lastname
+            }
+            ... on Organization {
+                x_opencti_organization_type
+            }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
+        """
+
+    @staticmethod
+    def generate_id(name, identity_class):
+        data = {"name": name.lower().strip(), "identity_class": identity_class.lower()}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "identity--" + id
+
+    @staticmethod
+    def generate_id_from_data(data):
+        return Identity.generate_id(data["name"], data["identity_class"])
 
     """
         List Identity objects
@@ -130,20 +257,23 @@ class Identity:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 500
+        with_files = kwargs.get("withFiles", False)
 
-        self.opencti.log(
-            "info", "Listing Identities with filters " + json.dumps(filters) + "."
+        self.opencti.app_logger.info(
+            "Listing Identities with filters", {"filters": json.dumps(filters)}
         )
         query = (
             """
-            query Identities($types: [String], $filters: [IdentitiesFiltering], $search: String, $first: Int, $after: ID, $orderBy: IdentitiesOrdering, $orderMode: OrderingMode) {
+            query Identities($types: [String], $filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: IdentitiesOrdering, $orderMode: OrderingMode) {
                 identities(types: $types, filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -170,13 +300,35 @@ class Identity:
                 "orderMode": order_mode,
             },
         )
-        return self.opencti.process_multiple(
-            result["data"]["identities"], with_pagination
-        )
+        if get_all:
+            final_data = []
+            data = self.opencti.process_multiple(result["data"]["identities"])
+            final_data = final_data + data
+            while result["data"]["identities"]["pageInfo"]["hasNextPage"]:
+                after = result["data"]["identities"]["pageInfo"]["endCursor"]
+                self.opencti.app_logger.info("Listing Identities", {"after": after})
+                result = self.opencti.query(
+                    query,
+                    {
+                        "filters": filters,
+                        "search": search,
+                        "first": first,
+                        "after": after,
+                        "orderBy": order_by,
+                        "orderMode": order_mode,
+                    },
+                )
+                data = self.opencti.process_multiple(result["data"]["identities"])
+                final_data = final_data + data
+            return final_data
+        else:
+            return self.opencti.process_multiple(
+                result["data"]["identities"], with_pagination
+            )
 
     """
         Read a Identity object
-        
+
         :param id: the id of the Identity
         :param filters: the filters to apply if no id provided
         :return Identity object
@@ -186,8 +338,9 @@ class Identity:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            self.opencti.log("info", "Reading Identity {" + id + "}.")
+            self.opencti.app_logger.info("Reading Identity", {"id": id})
             query = (
                 """
                 query Identity($id: String!) {
@@ -196,7 +349,7 @@ class Identity:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -212,8 +365,8 @@ class Identity:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_identity] Missing parameters: id or filters"
+            self.opencti.app_logger.error(
+                "[opencti_identity] Missing parameters: id or filters"
             )
             return None
 
@@ -237,7 +390,7 @@ class Identity:
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
         name = kwargs.get("name", None)
-        description = kwargs.get("description", "")
+        description = kwargs.get("description", None)
         contact_information = kwargs.get("contact_information", None)
         roles = kwargs.get("roles", None)
         x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
@@ -245,10 +398,13 @@ class Identity:
         x_opencti_reliability = kwargs.get("x_opencti_reliability", None)
         x_opencti_firstname = kwargs.get("x_opencti_firstname", None)
         x_opencti_lastname = kwargs.get("x_opencti_lastname", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
-        if name is not None and description is not None:
-            self.opencti.log("info", "Creating Identity {" + name + "}.")
+        if type is not None and name is not None:
+            self.opencti.app_logger.info("Creating Identity", {"name": name})
             input_variables = {
                 "stix_id": stix_id,
                 "createdBy": created_by,
@@ -265,11 +421,13 @@ class Identity:
                 "contact_information": contact_information,
                 "roles": roles,
                 "x_opencti_aliases": x_opencti_aliases,
+                "x_opencti_stix_ids": x_opencti_stix_ids,
+                "x_opencti_workflow_id": x_opencti_workflow_id,
                 "update": update,
             }
             if type == IdentityTypes.ORGANIZATION.value:
                 query = """
-                    mutation OrganizationAdd($input: OrganizationAddInput) {
+                    mutation OrganizationAdd($input: OrganizationAddInput!) {
                         organizationAdd(input: $input) {
                             id
                             standard_id
@@ -278,14 +436,14 @@ class Identity:
                         }
                     }
                 """
-                input_variables[
-                    "x_opencti_organization_type"
-                ] = x_opencti_organization_type
+                input_variables["x_opencti_organization_type"] = (
+                    x_opencti_organization_type
+                )
                 input_variables["x_opencti_reliability"] = x_opencti_reliability
                 result_data_field = "organizationAdd"
             elif type == IdentityTypes.INDIVIDUAL.value:
                 query = """
-                    mutation IndividualAdd($input: IndividualAddInput) {
+                    mutation IndividualAdd($input: IndividualAddInput!) {
                         individualAdd(input: $input) {
                             id
                             standard_id
@@ -294,12 +452,30 @@ class Identity:
                         }
                     }
                 """
+                input_variables["objectOrganization"] = granted_refs
                 input_variables["x_opencti_firstname"] = x_opencti_firstname
                 input_variables["x_opencti_lastname"] = x_opencti_lastname
+                input_variables["x_opencti_reliability"] = x_opencti_reliability
                 result_data_field = "individualAdd"
+            elif type == IdentityTypes.SYSTEM.value:
+                query = """
+                    mutation SystemAdd($input: SystemAddInput!) {
+                        systemAdd(input: $input) {
+                            id
+                            standard_id
+                            entity_type
+                            parent_types
+                        }
+                    }
+                """
+                input_variables["objectOrganization"] = granted_refs
+                input_variables["x_opencti_firstname"] = x_opencti_firstname
+                input_variables["x_opencti_lastname"] = x_opencti_lastname
+                input_variables["x_opencti_reliability"] = x_opencti_reliability
+                result_data_field = "systemAdd"
             else:
                 query = """
-                    mutation IdentityAdd($input: IdentityAddInput) {
+                    mutation IdentityAdd($input: IdentityAddInput!) {
                         identityAdd(input: $input) {
                             id
                             standard_id
@@ -320,7 +496,9 @@ class Identity:
                 result["data"][result_data_field]
             )
         else:
-            self.opencti.log("error", "Missing parameters: name and description")
+            self.opencti.app_logger.error(
+                "Missing parameters: type, name and description"
+            )
 
     """
         Import an Identity object from a STIX2 object
@@ -340,56 +518,130 @@ class Identity:
                     type = "Individual"
                 elif stix_object["identity_class"] == "class":
                     type = "Sector"
+                elif stix_object["identity_class"] == "system":
+                    type = "System"
+
+            # Search in extensions
+            if "x_opencti_aliases" not in stix_object:
+                stix_object["x_opencti_aliases"] = (
+                    self.opencti.get_attribute_in_extension("aliases", stix_object)
+                )
+            if "x_opencti_organization_type" not in stix_object:
+                stix_object["x_opencti_organization_type"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "organization_type", stix_object
+                    )
+                )
+            if "x_opencti_reliability" not in stix_object:
+                stix_object["x_opencti_reliability"] = (
+                    self.opencti.get_attribute_in_extension("reliability", stix_object)
+                )
+            if "x_opencti_organization_type" not in stix_object:
+                stix_object["x_opencti_organization_type"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "organization_type", stix_object
+                    )
+                )
+            if "x_opencti_firstname" not in stix_object:
+                stix_object["x_opencti_firstname"] = (
+                    self.opencti.get_attribute_in_extension("firstname", stix_object)
+                )
+            if "x_opencti_lastname" not in stix_object:
+                stix_object["x_opencti_lastname"] = (
+                    self.opencti.get_attribute_in_extension("lastname", stix_object)
+                )
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
+                )
+
             return self.create(
                 type=type,
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else [],
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=stix_object["name"],
-                description=self.opencti.stix2.convert_markdown(
-                    stix_object["description"]
-                )
-                if "description" in stix_object
-                else "",
-                contact_information=self.opencti.stix2.convert_markdown(
-                    stix_object["contact_information"]
-                )
-                if "contact_information" in stix_object
-                else None,
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_object["description"])
+                    if "description" in stix_object
+                    else None
+                ),
+                contact_information=(
+                    self.opencti.stix2.convert_markdown(
+                        stix_object["contact_information"]
+                    )
+                    if "contact_information" in stix_object
+                    else None
+                ),
                 roles=stix_object["roles"] if "roles" in stix_object else None,
                 x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
-                x_opencti_organization_type=stix_object["x_opencti_organization_type"]
-                if "x_opencti_organization_type" in stix_object
-                else None,
-                x_opencti_reliability=stix_object["x_opencti_reliability"]
-                if "x_opencti_reliability" in stix_object
-                else None,
-                x_opencti_firstname=stix_object["x_opencti_firstname"]
-                if "x_opencti_firstname" in stix_object
-                else None,
-                x_opencti_lastname=stix_object["x_opencti_lastname"]
-                if "x_opencti_lastname" in stix_object
-                else None,
+                x_opencti_organization_type=(
+                    stix_object["x_opencti_organization_type"]
+                    if "x_opencti_organization_type" in stix_object
+                    else None
+                ),
+                x_opencti_reliability=(
+                    stix_object["x_opencti_reliability"]
+                    if "x_opencti_reliability" in stix_object
+                    else None
+                ),
+                x_opencti_firstname=(
+                    stix_object["x_opencti_firstname"]
+                    if "x_opencti_firstname" in stix_object
+                    else None
+                ),
+                x_opencti_lastname=(
+                    stix_object["x_opencti_lastname"]
+                    if "x_opencti_lastname" in stix_object
+                    else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_identity] Missing parameters: stixObject"
+            self.opencti.app_logger.error(
+                "[opencti_identity] Missing parameters: stixObject"
             )
