@@ -5,8 +5,6 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class Language:
     def __init__(self, opencti):
@@ -35,13 +33,9 @@ class Language:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -54,28 +48,123 @@ class Language:
                 }
             }
             objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
                 edges {
                     node {
                         id
                         standard_id
                         entity_type
-                        definition_type
-                        definition
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
                         created
                         modified
-                        x_opencti_order
-                        x_opencti_color
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
-            objectLabel {
+            revoked
+            confidence
+            created
+            modified
+            name
+            aliases
+            importFiles {
                 edges {
                     node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
+        """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    created
+                    modified
+                    objectLabel {
                         id
                         value
                         color
                     }
                 }
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -135,6 +224,10 @@ class Language:
         id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
         return "language--" + id
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return Language.generate_id(data["name"])
+
     """
         List Language objects
 
@@ -155,18 +248,23 @@ class Language:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 100
+        with_files = kwargs.get("withFiles", False)
 
-        LOGGER.info("Listing Languages with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Languages with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-            query Languages($filters: [LanguagesFiltering], $search: String, $first: Int, $after: ID, $orderBy: LanguagesOrdering, $orderMode: OrderingMode) {
+            query Languages($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: LanguagesOrdering, $orderMode: OrderingMode) {
                 languages(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -198,7 +296,7 @@ class Language:
             final_data = final_data + data
             while result["data"]["languages"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["languages"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Languages after " + after)
+                self.opencti.app_logger.info("Listing Languages", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -230,8 +328,9 @@ class Language:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            LOGGER.info("Reading Language {%s}.", id)
+            self.opencti.app_logger.info("Reading Language", {"id": id})
             query = (
                 """
                 query Language($id: String!) {
@@ -240,7 +339,7 @@ class Language:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -256,7 +355,9 @@ class Language:
             else:
                 return None
         else:
-            LOGGER.error("[opencti_language] Missing parameters: id or filters")
+            self.opencti.app_logger.error(
+                "[opencti_language] Missing parameters: id or filters"
+            )
             return None
 
     """
@@ -283,7 +384,7 @@ class Language:
         update = kwargs.get("update", False)
 
         if name is not None:
-            LOGGER.info("Creating Language {%s}.", name)
+            self.opencti.app_logger.info("Creating Language", {"name": name})
             query = """
                 mutation LanguageAdd($input: LanguageAddInput!) {
                     languageAdd(input: $input) {
@@ -317,7 +418,7 @@ class Language:
             )
             return self.opencti.process_multiple_fields(result["data"]["languageAdd"])
         else:
-            LOGGER.error("[opencti_language] Missing parameters: name")
+            self.opencti.app_logger.error("[opencti_language] Missing parameters: name")
 
     """
         Import an Language object from a STIX2 object
@@ -333,41 +434,49 @@ class Language:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
                 )
 
             return self.opencti.language.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=stix_object["name"],
                 aliases=self.opencti.stix2.pick_aliases(stix_object),
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_language] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_language] Missing parameters: stixObject"
+            )
