@@ -5,8 +5,6 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class Malware:
     def __init__(self, opencti):
@@ -35,13 +33,9 @@ class Malware:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -53,29 +47,129 @@ class Malware:
                     x_opencti_lastname
                 }
             }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
             objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
                 edges {
                     node {
                         id
                         standard_id
                         entity_type
-                        definition_type
-                        definition
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
                         created
                         modified
-                        x_opencti_order
-                        x_opencti_color
                     }
                 }
             }
-            objectLabel {
-                edges {
-                    node {
+            revoked
+            confidence
+            created
+            modified
+            name
+            description
+            aliases
+            malware_types
+            is_family
+            first_seen
+            last_seen
+            architecture_execution_envs
+            implementation_languages
+            capabilities
+            killChainPhases {
+              id
+              standard_id
+              entity_type
+              kill_chain_name
+              phase_name
+              x_opencti_order
+              created
+              modified
+            }
+            samples {
+                id
+            }
+        """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    created
+                    modified
+                    objectLabel {
                         id
                         value
                         color
                     }
                 }
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -121,18 +215,17 @@ class Malware:
             implementation_languages
             capabilities
             killChainPhases {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        kill_chain_name
-                        phase_name
-                        x_opencti_order
-                        created
-                        modified
-                    }
-                }
+              id
+              standard_id
+              entity_type
+              kill_chain_name
+              phase_name
+              x_opencti_order
+              created
+              modified
+            }
+            samples {
+                id
             }
             importFiles {
                 edges {
@@ -157,6 +250,10 @@ class Malware:
         id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
         return "malware--" + id
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return Malware.generate_id(data["name"])
+
     """
         List Malware objects
 
@@ -177,18 +274,23 @@ class Malware:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 500
+        with_files = kwargs.get("withFiles", False)
 
-        LOGGER.info("Listing Malwares with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Malwares with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-            query Malwares($filters: [MalwaresFiltering], $search: String, $first: Int, $after: ID, $orderBy: MalwaresOrdering, $orderMode: OrderingMode) {
+            query Malwares($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: MalwaresOrdering, $orderMode: OrderingMode) {
                 malwares(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -221,7 +323,7 @@ class Malware:
             final_data = final_data + data
             while result["data"]["malwares"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["malwares"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Malwares after %s", after)
+                self.opencti.app_logger.info("Listing Malwares", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -253,8 +355,9 @@ class Malware:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            LOGGER.info("Reading Malware {%s}.", id)
+            self.opencti.app_logger.info("Reading Malware", {"id": id})
             query = (
                 """
                 query Malware($id: String!) {
@@ -263,7 +366,7 @@ class Malware:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -279,7 +382,9 @@ class Malware:
             else:
                 return None
         else:
-            LOGGER.error("[opencti_malware] Missing parameters: id or filters")
+            self.opencti.app_logger.error(
+                "[opencti_malware] Missing parameters: id or filters"
+            )
             return None
 
     """
@@ -301,7 +406,7 @@ class Malware:
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
         name = kwargs.get("name", None)
-        description = kwargs.get("description", "")
+        description = kwargs.get("description", None)
         aliases = kwargs.get("aliases", None)
         malware_types = kwargs.get("malware_types", None)
         is_family = kwargs.get("is_family", None)
@@ -313,12 +418,14 @@ class Malware:
         kill_chain_phases = kwargs.get("killChainPhases", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
+        samples = kwargs.get("samples", None)
         update = kwargs.get("update", False)
 
-        if name is not None and description is not None:
-            LOGGER.info("Creating Malware {%s}.", name)
+        if name is not None:
+            self.opencti.app_logger.info("Creating Malware", {"name": name})
             query = """
-                mutation MalwareAdd($input: MalwareAddInput) {
+                mutation MalwareAdd($input: MalwareAddInput!) {
                     malwareAdd(input: $input) {
                         id
                         standard_id
@@ -354,13 +461,17 @@ class Malware:
                         "capabilities": capabilities,
                         "killChainPhases": kill_chain_phases,
                         "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "x_opencti_workflow_id": x_opencti_workflow_id,
+                        "samples": samples,
                         "update": update,
                     }
                 },
             )
             return self.opencti.process_multiple_fields(result["data"]["malwareAdd"])
         else:
-            LOGGER.error("[opencti_malware] Missing parameters: name and description")
+            self.opencti.app_logger.error(
+                "[opencti_malware] Missing parameters: name and description"
+            )
 
     """
         Import an Malware object from a STIX2 object
@@ -376,73 +487,103 @@ class Malware:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                 )
 
             return self.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=stix_object["name"],
-                description=self.opencti.stix2.convert_markdown(
-                    stix_object["description"]
-                )
-                if "description" in stix_object
-                else "",
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_object["description"])
+                    if "description" in stix_object
+                    else None
+                ),
                 aliases=self.opencti.stix2.pick_aliases(stix_object),
-                malware_types=stix_object["malware_types"]
-                if "malware_types" in stix_object
-                else None,
-                is_family=stix_object["is_family"]
-                if "is_family" in stix_object
-                else False,
-                first_seen=stix_object["first_seen"]
-                if "first_seen" in stix_object
-                else None,
-                last_seen=stix_object["last_seen"]
-                if "last_seen" in stix_object
-                else None,
-                architecture_execution_envs=stix_object["architecture_execution_envs"]
-                if "architecture_execution_envs" in stix_object
-                else None,
-                implementation_languages=stix_object["implementation_languages"]
-                if "implementation_languages" in stix_object
-                else None,
-                capabilities=stix_object["capabilities"]
-                if "capabilities" in stix_object
-                else None,
-                killChainPhases=extras["kill_chain_phases_ids"]
-                if "kill_chain_phases_ids" in extras
-                else None,
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
-                objectOrganization=stix_object["granted_refs"]
-                if "granted_refs" in stix_object
-                else None,
+                malware_types=(
+                    stix_object["malware_types"]
+                    if "malware_types" in stix_object
+                    else None
+                ),
+                is_family=(
+                    stix_object["is_family"] if "is_family" in stix_object else False
+                ),
+                first_seen=(
+                    stix_object["first_seen"] if "first_seen" in stix_object else None
+                ),
+                last_seen=(
+                    stix_object["last_seen"] if "last_seen" in stix_object else None
+                ),
+                architecture_execution_envs=(
+                    stix_object["architecture_execution_envs"]
+                    if "architecture_execution_envs" in stix_object
+                    else None
+                ),
+                implementation_languages=(
+                    stix_object["implementation_languages"]
+                    if "implementation_languages" in stix_object
+                    else None
+                ),
+                capabilities=(
+                    stix_object["capabilities"]
+                    if "capabilities" in stix_object
+                    else None
+                ),
+                killChainPhases=(
+                    extras["kill_chain_phases_ids"]
+                    if "kill_chain_phases_ids" in extras
+                    else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
+                samples=(extras["sample_ids"] if "sample_ids" in extras else None),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_malware] Missing parameters: stixObject"
+            )
