@@ -6,8 +6,6 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class Note:
     def __init__(self, opencti):
@@ -36,13 +34,9 @@ class Note:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -54,29 +48,26 @@ class Note:
                     x_opencti_lastname
                 }
             }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -91,19 +82,6 @@ class Note:
                         external_id
                         created
                         modified
-                        importFiles {
-                            edges {
-                                node {
-                                    id
-                                    name
-                                    size
-                                    metaData {
-                                        mimetype
-                                        version
-                                    }
-                                }
-                            }
-                        }
                     }
                 }
             }
@@ -116,7 +94,7 @@ class Note:
             authors
             note_types
             likelihood
-            objects {
+            objects(all: true) {
                 edges {
                     node {
                         ... on BasicObject {
@@ -213,6 +191,9 @@ class Note:
                         ... on Case {
                             name
                         }
+                        ... on StixCyberObservable {
+                            observable_value
+                        }                        
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -220,6 +201,221 @@ class Note:
                             updated_at
                             relationship_type
                         }
+                       ... on StixSightingRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }                        
+                    }
+                }
+            }
+        """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    created
+                    modified
+                    objectLabel {
+                        id
+                        value
+                        color
+                    }
+                }
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
+                edges {
+                    node {
+                        id
+                        standard_id
+                        entity_type
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
+                        created
+                        modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            revoked
+            confidence
+            created
+            modified
+            attribute_abstract
+            content
+            authors
+            note_types
+            likelihood
+            objects(all: true) {
+                edges {
+                    node {
+                        ... on BasicObject {
+                            id
+                            entity_type
+                            parent_types
+                        }
+                        ... on BasicRelationship {
+                            id
+                            entity_type
+                            parent_types
+                        }
+                        ... on StixObject {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }
+                        ... on AttackPattern {
+                            name
+                        }
+                        ... on Campaign {
+                            name
+                        }
+                        ... on CourseOfAction {
+                            name
+                        }
+                        ... on Individual {
+                            name
+                        }
+                        ... on Organization {
+                            name
+                        }
+                        ... on Sector {
+                            name
+                        }
+                        ... on System {
+                            name
+                        }
+                        ... on Indicator {
+                            name
+                        }
+                        ... on Infrastructure {
+                            name
+                        }
+                        ... on IntrusionSet {
+                            name
+                        }
+                        ... on Position {
+                            name
+                        }
+                        ... on City {
+                            name
+                        }
+                        ... on Country {
+                            name
+                        }
+                        ... on Region {
+                            name
+                        }
+                        ... on Malware {
+                            name
+                        }
+                        ... on ThreatActor {
+                            name
+                        }
+                        ... on Tool {
+                            name
+                        }
+                        ... on Vulnerability {
+                            name
+                        }
+                        ... on Incident {
+                            name
+                        }
+                        ... on Event {
+                            name
+                        }
+                        ... on Channel {
+                            name
+                        }
+                        ... on Narrative {
+                            name
+                        }
+                        ... on Language {
+                            name
+                        }
+                        ... on DataComponent {
+                            name
+                        }
+                        ... on DataSource {
+                            name
+                        }
+                        ... on Case {
+                            name
+                        }
+                        ... on StixCyberObservable {
+                            observable_value
+                        }                        
+                        ... on StixCoreRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                            relationship_type
+                        }
+                       ... on StixSightingRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }                        
                     }
                 }
             }
@@ -240,14 +436,22 @@ class Note:
 
     @staticmethod
     def generate_id(created, content):
-        content = content.lower().strip()
-        if isinstance(created, datetime.datetime):
-            created = created.isoformat()
-        data = {"content": content, "created": created}
+        if content is None:
+            raise ValueError("content is required")
+        if created is not None:
+            if isinstance(created, datetime.datetime):
+                created = created.isoformat()
+            data = {"content": content, "created": created}
+        else:
+            data = {"content": content}
         data = canonicalize(data, utf8=False)
         id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
         return "note--" + id
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return Note.generate_id(data.get("created"), data["content"])
+
     """
         List Note objects
 
@@ -268,18 +472,23 @@ class Note:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 100
+        with_files = kwargs.get("withFiles", False)
 
-        LOGGER.info("Listing Notes with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Notes with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-            query Notes($filters: [NotesFiltering], $search: String, $first: Int, $after: ID, $orderBy: NotesOrdering, $orderMode: OrderingMode) {
+            query Notes($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: NotesOrdering, $orderMode: OrderingMode) {
                 notes(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -311,7 +520,7 @@ class Note:
             final_data = final_data + data
             while result["data"]["notes"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["notes"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Notes after " + after)
+                self.opencti.app_logger.info("Listing Notes", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -343,8 +552,9 @@ class Note:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            LOGGER.info("Reading Note {%s}.", id)
+            self.opencti.app_logger.info("Reading Note", {"id": id})
             query = (
                 """
                 query Note($id: String!) {
@@ -353,7 +563,7 @@ class Note:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -381,9 +591,12 @@ class Note:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Checking StixObjectOrStixRelationship {%s} in {%s}",
-                *(stix_object_or_stix_relationship_id, Note),
+            self.opencti.app_logger.info(
+                "Checking StixObjectOrStixRelationship in Note",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
                 query NoteContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -399,7 +612,9 @@ class Note:
             )
             return result["data"]["noteContainsStixObjectOrStixRelationship"]
         else:
-            LOGGER.error("[opencti_note] Missing parameters: id or entity_id")
+            self.opencti.app_logger.error(
+                "[opencti_note] Missing parameters: id or entity_id"
+            )
 
     """
         Create a Note object
@@ -427,12 +642,13 @@ class Note:
         likelihood = kwargs.get("likelihood", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
         if content is not None:
-            LOGGER.info("Creating Note {%s}.", content)
+            self.opencti.app_logger.info("Creating Note", {"content": content})
             query = """
-                mutation NoteAdd($input: NoteAddInput) {
+                mutation NoteAdd($input: NoteAddInput!) {
                     noteAdd(input: $input) {
                         id
                         standard_id
@@ -463,13 +679,14 @@ class Note:
                         "note_types": note_types,
                         "likelihood": likelihood,
                         "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "x_opencti_workflow_id": x_opencti_workflow_id,
                         "update": update,
                     }
                 },
             )
             return self.opencti.process_multiple_fields(result["data"]["noteAdd"])
         else:
-            LOGGER.error("[opencti_note] Missing parameters: content")
+            self.opencti.app_logger.error("[opencti_note] Missing parameters: content")
 
     """
         Add a Stix-Entity object to Note object (object_refs)
@@ -490,12 +707,15 @@ class Note:
                 stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
             ):
                 return True
-            LOGGER.info(
-                "Adding StixObjectOrStixRelationship {%s} to Note {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Adding StixObjectOrStixRelationship to Note",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
-               mutation NoteEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation NoteEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                    noteEdit(id: $id) {
                         relationAdd(input: $input) {
                             id
@@ -515,7 +735,7 @@ class Note:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_note] Missing parameters: id and stix_object_or_stix_relationship_id",
             )
             return False
@@ -534,9 +754,12 @@ class Note:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Removing StixObjectOrStixRelationship {%s} to {%s}",
-                *(stix_object_or_stix_relationship_id, Note),
+            self.opencti.app_logger.info(
+                "Removing StixObjectOrStixRelationship in Note",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
                mutation NotetEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -557,7 +780,9 @@ class Note:
             )
             return True
         else:
-            LOGGER.error("[opencti_note] Missing parameters: id and entity_id")
+            self.opencti.app_logger.error(
+                "[opencti_note] Missing parameters: id and entity_id"
+            )
             return False
 
     """
@@ -574,56 +799,79 @@ class Note:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                 )
 
             return self.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
                 objects=extras["object_ids"] if "object_ids" in extras else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
-                abstract=self.opencti.stix2.convert_markdown(stix_object["abstract"])
-                if "abstract" in stix_object
-                else "",
-                content=self.opencti.stix2.convert_markdown(stix_object["content"])
-                if "content" in stix_object
-                else "",
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
+                abstract=(
+                    self.opencti.stix2.convert_markdown(stix_object["abstract"])
+                    if "abstract" in stix_object
+                    else None
+                ),
+                content=(
+                    self.opencti.stix2.convert_markdown(stix_object["content"])
+                    if "content" in stix_object
+                    else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
                 authors=stix_object["authors"] if "authors" in stix_object else None,
-                note_types=stix_object["note_types"]
-                if "note_types" in stix_object
-                else None,
-                likelihood=stix_object["likelihood"]
-                if "likelihood" in stix_object
-                else None,
-                objectOrganization=stix_object["granted_refs"]
-                if "granted_refs" in stix_object
-                else None,
+                note_types=(
+                    stix_object["note_types"] if "note_types" in stix_object else None
+                ),
+                likelihood=(
+                    stix_object["likelihood"] if "likelihood" in stix_object else None
+                ),
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_note] Missing parameters: stixObject"
+            )
