@@ -1,9 +1,9 @@
 # coding: utf-8
-
+import datetime
 import json
 import uuid
 
-from pycti.entities import LOGGER
+from stix2.canonicalization.Canonicalize import canonicalize
 
 
 class Opinion:
@@ -33,13 +33,9 @@ class Opinion:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -52,28 +48,20 @@ class Opinion:
                 }
             }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -111,7 +99,7 @@ class Opinion:
             explanation
             authors
             opinion
-            objects {
+            objects(all: true) {
                 edges {
                     node {
                         ... on BasicObject {
@@ -189,7 +177,10 @@ class Opinion:
                         }
                         ... on Case {
                             name
-                        }                        
+                        }
+                        ... on StixCyberObservable {
+                            observable_value
+                        }                               
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -197,6 +188,12 @@ class Opinion:
                             updated_at
                             relationship_type
                         }
+                       ... on StixSightingRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }                         
                     }
                 }
             }
@@ -216,8 +213,22 @@ class Opinion:
         """
 
     @staticmethod
-    def generate_id():
-        return "opinion--" + str(uuid.uuid4())
+    def generate_id(created, opinion):
+        if opinion is None:
+            raise ValueError("opinion is required")
+        if created is not None:
+            if isinstance(created, datetime.datetime):
+                created = created.isoformat()
+            data = {"opinion": opinion, "created": created}
+        else:
+            data = {"opinion": opinion}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "opinion--" + id
+
+    @staticmethod
+    def generate_id_from_data(data):
+        return Opinion.generate_id(data.get("created"), data["opinion"])
 
     """
         List Opinion objects
@@ -239,13 +250,13 @@ class Opinion:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 100
 
-        LOGGER.info("Listing Opinions with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Opinions with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-            query Opinions($filters: [OpinionsFiltering], $search: String, $first: Int, $after: ID, $orderBy: OpinionsOrdering, $orderMode: OrderingMode) {
+            query Opinions($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: OpinionsOrdering, $orderMode: OrderingMode) {
                 opinions(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
@@ -282,7 +293,7 @@ class Opinion:
             final_data = final_data + data
             while result["data"]["opinions"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["opinions"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Opinions after " + after)
+                self.opencti.app_logger.info("Listing Opinions", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -315,7 +326,7 @@ class Opinion:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            LOGGER.info("Reading Opinion {%s}.", id)
+            self.opencti.app_logger.info("Reading Opinion", {"id": id})
             query = (
                 """
                 query Opinion($id: String!) {
@@ -352,9 +363,12 @@ class Opinion:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Checking StixObjectOrStixRelationship {%s} in Opinion {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Checking StixObjectOrStixRelationship in Opinion",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
                 query OpinionContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -370,7 +384,9 @@ class Opinion:
             )
             return result["data"]["opinionContainsStixObjectOrStixRelationship"]
         else:
-            LOGGER.error("[opencti_opinion] Missing parameters: id or entity_id")
+            self.opencti.app_logger.error(
+                "[opencti_opinion] Missing parameters: id or entity_id"
+            )
 
     """
         Create a Opinion object
@@ -399,9 +415,9 @@ class Opinion:
         update = kwargs.get("update", False)
 
         if opinion is not None:
-            LOGGER.info("Creating Opinion {%s}.", opinion)
+            self.opencti.app_logger.info("Creating Opinion", {"opinion": opinion})
             query = """
-                mutation OpinionAdd($input: OpinionAddInput) {
+                mutation OpinionAdd($input: OpinionAddInput!) {
                     opinionAdd(input: $input) {
                         id
                         standard_id
@@ -436,7 +452,9 @@ class Opinion:
             )
             return self.opencti.process_multiple_fields(result["data"]["opinionAdd"])
         else:
-            LOGGER.error("[opencti_opinion] Missing parameters: content")
+            self.opencti.app_logger.error(
+                "[opencti_opinion] Missing parameters: content"
+            )
 
     """
         Add a Stix-Entity object to Opinion object (object_refs)
@@ -457,12 +475,15 @@ class Opinion:
                 stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
             ):
                 return True
-            LOGGER.info(
-                "Adding StixObjectOrStixRelationship {%s} to Opinion {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Adding StixObjectOrStixRelationship to Opinion",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
-               mutation OpinionEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation OpinionEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                    opinionEdit(id: $id) {
                         relationAdd(input: $input) {
                             id
@@ -482,7 +503,7 @@ class Opinion:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_opinion] Missing parameters: id and stix_object_or_stix_relationship_id",
             )
             return False
@@ -501,9 +522,12 @@ class Opinion:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            LOGGER.info(
-                "Removing StixObjectOrStixRelationship {%s} to Opinion {%s}",
-                *(stix_object_or_stix_relationship_id, id),
+            self.opencti.app_logger.info(
+                "Removing StixObjectOrStixRelationship to Opinion",
+                {
+                    "id": id,
+                    "stixObjectOrStixRelationshipId": stix_object_or_stix_relationship_id,
+                },
             )
             query = """
                mutation OpinionEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -524,7 +548,9 @@ class Opinion:
             )
             return True
         else:
-            LOGGER.error("[opencti_opinion] Missing parameters: id and entity_id")
+            self.opencti.app_logger.error(
+                "[opencti_opinion] Missing parameters: id and entity_id"
+            )
             return False
 
     """
@@ -541,52 +567,60 @@ class Opinion:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
                 )
 
             return self.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else None,
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
                 objects=extras["object_ids"] if "object_ids" in extras else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else None,
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
-                explanation=self.opencti.stix2.convert_markdown(
-                    stix_object["explanation"]
-                )
-                if "explanation" in stix_object
-                else None,
-                authors=self.opencti.stix2.convert_markdown(stix_object["authors"])
-                if "authors" in stix_object
-                else None,
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
+                explanation=(
+                    self.opencti.stix2.convert_markdown(stix_object["explanation"])
+                    if "explanation" in stix_object
+                    else None
+                ),
+                authors=stix_object["authors"] if "authors" in stix_object else None,
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
                 opinion=stix_object["opinion"] if "opinion" in stix_object else None,
-                objectOrganization=stix_object["granted_refs"]
-                if "granted_refs" in stix_object
-                else None,
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_opinion] Missing parameters: stixObject"
+            )
