@@ -336,6 +336,7 @@ class StixCoreRelationship:
             start_time = start_time.isoformat()
         if isinstance(stop_time, datetime.datetime):
             stop_time = stop_time.isoformat()
+
         if start_time is not None and stop_time is not None:
             data = {
                 "relationship_type": relationship_type,
@@ -376,6 +377,16 @@ class StixCoreRelationship:
         :return List of stix_core_relationship objects
     """
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return StixCoreRelationship.generate_id(
+            data["relationship_type"],
+            data["source_ref"],
+            data["target_ref"],
+            data.get("start_time"),
+            data.get("stop_time"),
+        )
+
     def list(self, **kwargs):
         from_or_to_id = kwargs.get("fromOrToId", None)
         element_with_target_types = kwargs.get("elementWithTargetTypes", None)
@@ -397,8 +408,6 @@ class StixCoreRelationship:
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         search = kwargs.get("search", None)
-        if get_all:
-            first = 100
 
         self.opencti.app_logger.info(
             "Listing stix_core_relationships",
@@ -599,6 +608,7 @@ class StixCoreRelationship:
         kill_chain_phases = kwargs.get("killChainPhases", None)
         granted_refs = kwargs.get("objectOrganization", None)
         x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
         self.opencti.app_logger.info(
@@ -642,6 +652,7 @@ class StixCoreRelationship:
                     "externalReferences": external_references,
                     "killChainPhases": kill_chain_phases,
                     "x_opencti_workflow_id": x_opencti_workflow_id,
+                    "x_opencti_stix_ids": x_opencti_stix_ids,
                     "update": update,
                 }
             },
@@ -1132,6 +1143,10 @@ class StixCoreRelationship:
         default_date = kwargs.get("defaultDate", False)
         if stix_relation is not None:
             # Search in extensions
+            if "x_opencti_stix_ids" not in stix_relation:
+                stix_relation["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_relation)
+                )
             if "x_opencti_granted_refs" not in stix_relation:
                 stix_relation["x_opencti_granted_refs"] = (
                     self.opencti.get_attribute_in_extension(
@@ -1213,6 +1228,11 @@ class StixCoreRelationship:
                     if "x_opencti_workflow_id" in stix_relation
                     else None
                 ),
+                x_opencti_stix_ids=(
+                    stix_relation["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_relation
+                    else None
+                ),
                 update=update,
             )
         else:
