@@ -5,8 +5,6 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class StixSightingRelationship:
     def __init__(self, opencti):
@@ -42,13 +40,9 @@ class StixSightingRelationship:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -61,28 +55,25 @@ class StixSightingRelationship:
                 }
             }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -271,7 +262,12 @@ class StixSightingRelationship:
         """
 
     @staticmethod
-    def generate_id(source_ref, target_ref, first_seen=None, last_seen=None):
+    def generate_id(
+        sighting_of_ref,
+        where_sighted_refs,
+        first_seen=None,
+        last_seen=None,
+    ):
         if isinstance(first_seen, datetime.datetime):
             first_seen = first_seen.isoformat()
         if isinstance(last_seen, datetime.datetime):
@@ -279,20 +275,38 @@ class StixSightingRelationship:
 
         if first_seen is not None and last_seen is not None:
             data = {
-                "source_ref": source_ref,
-                "target_ref": target_ref,
+                "type": "sighting",
+                "sighting_of_ref": sighting_of_ref,
+                "where_sighted_refs": where_sighted_refs,
                 "first_seen": first_seen,
                 "last_seen": last_seen,
             }
+        elif first_seen is not None:
+            data = {
+                "type": "sighting",
+                "sighting_of_ref": sighting_of_ref,
+                "where_sighted_refs": where_sighted_refs,
+                "first_seen": first_seen,
+            }
         else:
             data = {
-                "source_ref": source_ref,
-                "target_ref": target_ref,
+                "type": "sighting",
+                "sighting_of_ref": sighting_of_ref,
+                "where_sighted_refs": where_sighted_refs,
             }
         data = canonicalize(data, utf8=False)
         id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
         return "sighting--" + id
 
+    @staticmethod
+    def generate_id_from_data(data):
+        return StixSightingRelationship.generate_id(
+            data["sighting_of_ref"],
+            data["where_sighted_refs"],
+            data.get("first_seen"),
+            data.get("last_seen"),
+        )
+
     """
         List stix_sightings objects
 
@@ -308,7 +322,7 @@ class StixSightingRelationship:
     """
 
     def list(self, **kwargs):
-        element_id = kwargs.get("elementId", None)
+        from_or_to_id = kwargs.get("fromOrToId", None)
         from_id = kwargs.get("fromId", None)
         from_types = kwargs.get("fromTypes", None)
         to_id = kwargs.get("toId", None)
@@ -317,7 +331,7 @@ class StixSightingRelationship:
         first_seen_stop = kwargs.get("firstSeenStop", None)
         last_seen_start = kwargs.get("lastSeenStart", None)
         last_seen_stop = kwargs.get("lastSeenStop", None)
-        filters = kwargs.get("filters", [])
+        filters = kwargs.get("filters", None)
         first = kwargs.get("first", 100)
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
@@ -325,17 +339,16 @@ class StixSightingRelationship:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
-        if get_all:
-            first = 100
+        search = kwargs.get("search", None)
 
-        LOGGER.info(
-            "Listing stix_sighting with {type: stix_sighting, from_id: %s, to_id: %s}",
-            *(from_id, to_id),
+        self.opencti.app_logger.info(
+            "Listing stix_sighting with {type: stix_sighting}",
+            {"from_id": from_id, "to_id": to_id},
         )
         query = (
             """
-                query StixSightingRelationships($elementId: String, $fromId: StixRef, $fromTypes: [String], $toId: StixRef, $toTypes: [String], $firstSeenStart: DateTime, $firstSeenStop: DateTime, $lastSeenStart: DateTime, $lastSeenStop: DateTime, $filters: [StixSightingRelationshipsFiltering], $first: Int, $after: ID, $orderBy: StixSightingRelationshipsOrdering, $orderMode: OrderingMode) {
-                    stixSightingRelationships(elementId: $elementId, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, firstSeenStart: $firstSeenStart, firstSeenStop: $firstSeenStop, lastSeenStart: $lastSeenStart, lastSeenStop: $lastSeenStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
+                query StixSightingRelationships($fromOrToId: String, $fromId: StixRef, $fromTypes: [String], $toId: StixRef, $toTypes: [String], $firstSeenStart: DateTime, $firstSeenStop: DateTime, $lastSeenStart: DateTime, $lastSeenStop: DateTime, $filters: FilterGroup, $first: Int, $after: ID, $orderBy: StixSightingRelationshipsOrdering, $orderMode: OrderingMode, $search: String) {
+                    stixSightingRelationships(fromOrToId: $fromOrToId, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, firstSeenStart: $firstSeenStart, firstSeenStop: $firstSeenStop, lastSeenStart: $lastSeenStart, lastSeenStop: $lastSeenStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode, search: $search) {
                         edges {
                             node {
                                 """
@@ -357,7 +370,7 @@ class StixSightingRelationship:
         result = self.opencti.query(
             query,
             {
-                "elementId": element_id,
+                "fromOrToId": from_or_to_id,
                 "fromId": from_id,
                 "fromTypes": from_types,
                 "toId": to_id,
@@ -371,6 +384,7 @@ class StixSightingRelationship:
                 "after": after,
                 "orderBy": order_by,
                 "orderMode": order_mode,
+                "search": search,
             },
         )
         if get_all:
@@ -385,11 +399,13 @@ class StixSightingRelationship:
                 after = result["data"]["stixSightingRelationships"]["pageInfo"][
                     "endCursor"
                 ]
-                LOGGER.info("Listing StixSightingRelationships after " + after)
+                self.opencti.app_logger.info(
+                    "Listing StixSightingRelationships", {"after": after}
+                )
                 result = self.opencti.query(
                     query,
                     {
-                        "elementId": element_id,
+                        "fromOrToId": from_or_to_id,
                         "fromId": from_id,
                         "fromTypes": from_types,
                         "toId": to_id,
@@ -430,7 +446,7 @@ class StixSightingRelationship:
 
     def read(self, **kwargs):
         id = kwargs.get("id", None)
-        element_id = kwargs.get("elementId", None)
+        from_or_to_id = kwargs.get("fromOrToId", None)
         from_id = kwargs.get("fromId", None)
         to_id = kwargs.get("toId", None)
         first_seen_start = kwargs.get("firstSeenStart", None)
@@ -438,8 +454,9 @@ class StixSightingRelationship:
         last_seen_start = kwargs.get("lastSeenStart", None)
         last_seen_stop = kwargs.get("lastSeenStop", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        filters = kwargs.get("filters", None)
         if id is not None:
-            LOGGER.info("Reading stix_sighting {%s}.", id)
+            self.opencti.app_logger.info("Reading stix_sighting", {"id": id})
             query = (
                 """
                     query StixSightingRelationship($id: String!) {
@@ -459,9 +476,15 @@ class StixSightingRelationship:
             return self.opencti.process_multiple_fields(
                 result["data"]["stixSightingRelationship"]
             )
+        elif filters is not None:
+            result = self.list(filters=filters)
+            if len(result) > 0:
+                return result[0]
+            else:
+                return None
         elif from_id is not None and to_id is not None:
             result = self.list(
-                elementId=element_id,
+                fromOrToId=from_or_to_id,
                 fromId=from_id,
                 toId=to_id,
                 firstSeenStart=first_seen_start,
@@ -474,7 +497,7 @@ class StixSightingRelationship:
             else:
                 return None
         else:
-            LOGGER.error("Missing parameters: id or from_id and to_id")
+            self.opencti.app_logger.error("Missing parameters: id or from_id and to_id")
             return None
 
     """
@@ -501,9 +524,12 @@ class StixSightingRelationship:
         object_label = kwargs.get("objectLabel", None)
         external_references = kwargs.get("externalReferences", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
-        LOGGER.info("Creating stix_sighting {%s, %s}.", from_id, to_id)
+        self.opencti.app_logger.info(
+            "Creating stix_sighting", {"from_id": from_id, "to_id": to_id}
+        )
         query = """
                 mutation StixSightingRelationshipAdd($input: StixSightingRelationshipAddInput!) {
                     stixSightingRelationshipAdd(input: $input) {
@@ -534,6 +560,7 @@ class StixSightingRelationship:
                     "objectLabel": object_label,
                     "externalReferences": external_references,
                     "x_opencti_stix_ids": x_opencti_stix_ids,
+                    "x_opencti_workflow_id": x_opencti_workflow_id,
                     "update": update,
                 }
             },
@@ -554,7 +581,7 @@ class StixSightingRelationship:
         id = kwargs.get("id", None)
         input = kwargs.get("input", None)
         if id is not None and input is not None:
-            LOGGER.info("Updating stix_sighting {%s}.", id)
+            self.opencti.app_logger.info("Updating stix_sighting", {"id": id})
             query = """
                     mutation StixSightingRelationshipEdit($id: ID!, $input: [EditInput]!) {
                         stixSightingRelationshipEdit(id: $id) {
@@ -575,7 +602,7 @@ class StixSightingRelationship:
                 result["data"]["stixSightingRelationshipEdit"]["fieldPatch"]
             )
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_stix_sighting] Missing parameters: id and key and value"
             )
             return None
@@ -595,36 +622,34 @@ class StixSightingRelationship:
             custom_attributes = """
                 id
                 objectMarking {
-                    edges {
-                        node {
-                            id
-                            standard_id
-                            entity_type
-                            definition_type
-                            definition
-                            x_opencti_order
-                            x_opencti_color
-                            created
-                            modified
-                        }
-                    }
+                    id
+                    standard_id
+                    entity_type
+                    definition_type
+                    definition
+                    x_opencti_order
+                    x_opencti_color
+                    created
+                    modified
                 }
             """
             stix_core_relationship = self.read(
                 id=id, customAttributes=custom_attributes
             )
             if stix_core_relationship is None:
-                LOGGER.error("Cannot add Marking-Definition, entity not found")
+                self.opencti.app_logger.error(
+                    "Cannot add Marking-Definition, entity not found"
+                )
                 return False
             if marking_definition_id in stix_core_relationship["objectMarkingIds"]:
                 return True
             else:
-                LOGGER.info(
-                    "Adding Marking-Definition {%s} to stix_sighting_relationship {%s}",
-                    *(marking_definition_id, id),
+                self.opencti.app_logger.info(
+                    "Adding Marking-Definition to stix_sighting_relationship",
+                    {"marking_definition_id": marking_definition_id, "id": id},
                 )
                 query = """
-                   mutation StixSightingRelationshipEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+                   mutation StixSightingRelationshipEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                        stixSightingRelationshipEdit(id: $id) {
                             relationAdd(input: $input) {
                                 id
@@ -644,7 +669,9 @@ class StixSightingRelationship:
                 )
                 return True
         else:
-            LOGGER.error("Missing parameters: id and marking_definition_id")
+            self.opencti.app_logger.error(
+                "Missing parameters: id and marking_definition_id"
+            )
             return False
 
     """
@@ -659,9 +686,9 @@ class StixSightingRelationship:
         id = kwargs.get("id", None)
         marking_definition_id = kwargs.get("marking_definition_id", None)
         if id is not None and marking_definition_id is not None:
-            LOGGER.info(
-                "Removing Marking-Definition {%s} from stix_sighting_relationship {%s}",
-                *(marking_definition_id, id),
+            self.opencti.app_logger.info(
+                "Removing Marking-Definition from stix_sighting_relationship",
+                {"marking_definition_id": marking_definition_id, "id": id},
             )
             query = """
                mutation StixSightingRelationshipEdit($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -682,7 +709,7 @@ class StixSightingRelationship:
             )
             return True
         else:
-            LOGGER.error("Missing parameters: id and label_id")
+            self.opencti.app_logger.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -697,9 +724,9 @@ class StixSightingRelationship:
         id = kwargs.get("id", None)
         identity_id = kwargs.get("identity_id", None)
         if id is not None:
-            LOGGER.info(
-                "Updating author of stix_sighting_relationship {%s} with Identity {%s}",
-                *(id, identity_id),
+            self.opencti.app_logger.info(
+                "Updating author of stix_sighting_relationship with Identity",
+                {"id": id, "identity_id": identity_id},
             )
             custom_attributes = """
                 id
@@ -747,7 +774,7 @@ class StixSightingRelationship:
             if identity_id is not None:
                 # Add the new relation
                 query = """
-                    mutation StixSightingRelationshipEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+                    mutation StixSightingRelationshipEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                         stixSightingRelationshipEdit(id: $id) {
                             relationAdd(input: $input) {
                                 id
@@ -764,7 +791,7 @@ class StixSightingRelationship:
                 }
                 self.opencti.query(query, variables)
         else:
-            LOGGER.error("Missing parameters: id")
+            self.opencti.app_logger.error("Missing parameters: id")
             return False
 
     """
@@ -777,7 +804,7 @@ class StixSightingRelationship:
     def delete(self, **kwargs):
         id = kwargs.get("id", None)
         if id is not None:
-            LOGGER.info("Deleting stix_sighting {%s}.", id)
+            self.opencti.app_logger.info("Deleting stix_sighting", {"id": id})
             query = """
                 mutation StixSightingRelationshipEdit($id: ID!) {
                     stixSightingRelationshipEdit(id: $id) {
@@ -787,5 +814,7 @@ class StixSightingRelationship:
             """
             self.opencti.query(query, {"id": id})
         else:
-            LOGGER.error("[opencti_stix_sighting] Missing parameters: id")
+            self.opencti.app_logger.error(
+                "[opencti_stix_sighting] Missing parameters: id"
+            )
             return None
